library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package router_types is
    constant BUFF_SIZE_IN       :   positive  :=  8;
    constant BUFF_SIZE_OUT      :   positive  :=  4;
    constant AV_POS_SIZE        :   positive  :=  6;
    constant NUM_DIRS           :   positive  :=  5;
    constant NUM_DIRS_PARENT    :   positive  :=  8;
    constant DEST_ADDR_SIZE     :   positive  :=  5;
    constant CHIP_ADDR_SIZE     :   positive  :=  5; -- *
    constant MAX_PATHS_SIZE     :   positive  :=  96;
    constant MAX_EX_PATHS_SIZE  :   positive  :=  256;
    constant TOP_LEVEL          :   positive  :=  5;

    subtype WORD is std_logic_vector(63 downto 0);
    subtype H_WORD is std_logic_vector(31 downto 0);
    type t_PACKET_STATE is (Idle, Arbitration, InArbQueue, WritingFifo);
    type t_PACKET_STATES is array (natural range <>) of t_PACKET_STATE;
    type t_OUT_BUFF_RD_STATE is (EmptyFifo, StartRead, Sending, WaitForSender);
    type t_OUT_BUFF_RD_STATES is array (natural range <>) of t_OUT_BUFF_RD_STATE;
    type t_ARBITER_STATE is (DirDecoding, PathSelect, ArbComplete);

    type t_fifo_wr_in is record
        data        :   WORD;
        wr_req      :   std_logic;
    end record;

    type t_fifo_out is record
        data        :   WORD;
        empty       :   std_logic;
        full        :   std_logic;
    end record;

    type t_chip_addr is record
        x :   std_logic_vector(CHIP_ADDR_SIZE-1 downto 0);
        y :   std_logic_vector(CHIP_ADDR_SIZE-1 downto 0);
    end record;

    type t_core_addr is record
        x :   std_logic_vector(DEST_ADDR_SIZE-1 downto 1); -- last bit is always '-' for all routers
        y :   std_logic_vector(DEST_ADDR_SIZE-1 downto 1);
    end record;

    type t_addr is record
        x :   std_logic_vector(DEST_ADDR_SIZE-1 downto 0);
        y :   std_logic_vector(DEST_ADDR_SIZE-1 downto 0);
    end record;

    type t_pos_addr is record
        chip_pos :   t_chip_addr;
        core_pos :   t_addr;
        copy_pos :   t_addr;
    end record;

    type t_FIFO_WR_INS   is array (natural range <>) of t_fifo_wr_in;
    type t_FIFO_RD_INS   is array (natural range <>) of std_logic; -- rd_req
    type t_FIFO_OUTS     is array (natural range <>) of t_fifo_out;
    type t_DATA          is array (natural range <>) of WORD;
    type t_DATA_EXT      is array (natural range <>) of WORD;--H_WORD;
    type t_DATA_DIRS     is array (natural range <>) of
                            std_logic_vector(NUM_DIRS-1 downto 0);
    type t_DATA_DIRS_EXT is array (natural range <>) of
                            std_logic_vector(NUM_DIRS_PARENT-1 downto 0);

    constant DEF_FIFO_WR_IN : t_fifo_wr_in := (
        data    => (others => '0'),
        wr_req  => '0'
    );
end package;

package body router_types is
end package body;
  